<?php
/*
set_error_handler(
  function( $errno, $errstr, $errfile, $errline ) {
    throw new ErrorException( $errstr, $errno, 0, $errfile, $errline ) ;
  }
) ;

set_exception_handler(function($e) {
  echo "$e\n" ;
//  debug() ;
}) ;
*/

/* get array data safely */
function getArray( $arr, $key, $default=0 ) {
  if( array_key_exists("$key", $arr) ) {
    $ret = $arr[$key] ;
  } else {
    $ret = $default ;
  }
  return $ret ;
}

/* get data */
function getData( $arr, $key, $type, $minlen, $maxlen, $default=NULL ) {
  // get from array
  if( array_key_exists("$key", $arr) ) {
    $data = $arr[$key] ;
  } else {
    $data = $default ;
  }

  // filter
  $l1 = strlen( $data ) ;
  $filter = "" ;
  // alphabet
  if( $type & DATA_AL ) {
    $filter .= "a-zA-Z" ;
  } 
  // number
  if( $type & DATA_NUM ) {
    $filter .= "0-9" ;
  }
  if( strlen($filter) > 0 ) {
    $filter = sprintf( "/[^%s]/", $filter ) ;
    $data = preg_replace( $filter, '', $data ) ;
  }

  // check length
  $l2 = strlen( $data ) ;
  if( $l1 != $l2 || $l2 < $minlen || $l2 > $maxlen ) {
    $data = $default ;
  }

  return $data ;
}

function nowDate() {
  return date( 'Y/m/d H:i:s' ) ;
}

function printMsg( $msg ) {
  $buf = "" ;
  foreach( $msg as $s ) {
    $si = str_repeat( ">", $s[1] ) ;
    $buf .= sprintf( "%s %s %s\n", nowDate(), $si, $s[0] ) ;
  }
  printf( $buf ) ;
}

function isCommand( $cmd ) {
  return preg_match('/^[0-9A-Za-z]+ +$/', $cmd) ;
}

function makeCommand( $cmd ) {
  $fmt = "%-" . PACKET_SIZE_T . "s" ;
  return sprintf( $fmt, $cmd ) ;
}

function initSHM( $key, $size ) {
  $shmid = shmop_open( $key, $size, 'c', 0644, $size ) ;
  if( $shmid !== FALSE ) {
    shmop_delete( $shmid ) ;
    shmop_close( $shmid ) ;
  }
}

function writeSHM( $key, $size, $data ) {
  // write data to shared memory
  $shmid = @shmop_open( $key, 'n', 0644, $size ) ;
  if( $shmid !== FALSE ) {
    shmop_write( $shmid, $data, 0 ) ;
    shmop_close( $shmid ) ;
  }
}

function getWData( $ud ) {
  $wData = array(
    "m_time" => $ud["d01_1"],
    "m_totalDistance" => $ud["d01_4"],
    "m_drs" => $ud["d01_43"],
    "m_fuel_in_tank" => $ud["d01_46"],
    "m_fuel_capacity" => $ud["d01_47"],
    "m_brakes_temp" => array(
      $ud["d01_54"], $ud["d01_55"], $ud["d01_52"], $ud["d01_53"],
    ),
    "m_tyres_pressure" => array(
      $ud["d01_58"], $ud["d01_59"], $ud["d01_56"], $ud["d01_57"],
    ),
    "m_total_laps" => $ud["d01_61"],
    "m_track_size" => $ud["d01_62"],
    "m_sessionType" => $ud["d01_67"],
    "m_drsAllowed" => $ud["d01_68"],
    "m_track_number" => $ud["d01_69"],
    "m_vehicleFIAFlags" => $ud["d01_70"],
    "m_era" => $ud["d01_71"],
    "m_engine_temperature" => $ud["d01_72"],
    "m_tyres_temperature" => array(
      $ud["d02_3"], $ud["d02_4"], $ud["d02_1"], $ud["d02_2"],
    ),
    "m_tyres_wear" => array(
      $ud["d02_7"], $ud["d02_8"], $ud["d02_5"], $ud["d02_6"],
    ),
    "m_front_brake_bias" => $ud["d02_10"],
    "m_fuel_mix" => $ud["d02_11"],
    "m_tyres_damage" => array(
      $ud["d02_15"], $ud["d02_16"], $ud["d02_13"], $ud["d02_14"],
    ),
    "m_front_left_wing_damage" => $ud["d02_17"],
    "m_front_right_wing_damage" => $ud["d02_18"],
    "m_rear_wing_damage" => $ud["d02_19"],
    "m_engine_damage" => $ud["d02_20"],
    "m_gear_box_damage" => $ud["d02_21"],
    "m_exhaust_damage" => $ud["d02_22"],
    "m_pit_speed_limit" => $ud["d02_24"],
    "m_session_time_left" => $ud["d03_1"],
    "m_num_cars" => $ud["d04_4"],
    "m_player_car_index" => $ud["d04_5"],
  ) ;

  return $wData ;
}

// get cars data from D-BOX
function getCData( $ud, $m_num_cars ) {
  $cData = array() ;
  for( $i=0 ; $i<CARS_MAX ; $i++ ) {
    $key1 = sprintf("d%da_",$i+11 ) ;
    $key2 = sprintf("d%db_",$i+11 ) ;
    $cData[$i] = array(
      "m_lastLapTime" => $ud["${key1}4"],
      "m_currentLapTime" => $ud["${key1}5"],
      "m_bestLapTime" => $ud["${key1}6"],
      "m_sector1Time" => $ud["${key1}7"],
      "m_sector2Time" => $ud["${key1}8"],
      "m_lapDistance" => $ud["${key1}9"],
      "m_driverId" => $ud["${key2}1"],
      "m_teamId" => $ud["${key2}2"],
      "m_carPosition" => $ud["${key2}3"],
      "m_currentLapNum" => $ud["${key2}4"],
      "m_tyreCompound" => $ud["${key2}5"],
      "m_inPits" => $ud["${key2}6"],
      "m_sector" => $ud["${key2}7"],
      "m_currentLapInvalid" => $ud["${key2}8"],
      "m_penalties" => $ud["${key2}9"],
    ) ;
  }
  return $cData ;
}

function addDebugMsg( $str ) {
  global $DEBUG_BUF ;

  $DEBUG_BUF .= $str ;
}

function putDebugMsg() {
  global $DEBUG_MODE, $DEBUG_INIT, $DEBUG_BUF ;

  if( DEBUG_ON && $DEBUG_MODE != 0 && strlen($DEBUG_BUF) > 0 ) {
    printf( "%s", $DEBUG_BUF ) ;
    if( $DEBUG_INIT ) {
      file_put_contents( DEBUG_LOG, $DEBUG_BUF, FILE_APPEND ) ;
    } else {
      file_put_contents( DEBUG_LOG, $DEBUG_BUF ) ;
      $DEBUG_INIT = 1 ;
    }
    $DEBUG_BUF = "" ;
  }
}

/* debug */
function debug() {
  global $rpos, $wData, $cData, $wDataMst, $cDataMst ;
  echo "pos: " ;
  print_r( $rpos ) ;
  echo "wData: " ;
  print_r( $wData ) ;
  echo "cData: " ;
  print_r( $cData ) ;
  echo "wDataMst: " ;
  print_r( $wDataMst ) ;
  echo "cDataMst: " ;
  print_r( $cDataMst ) ;
/*
  debugSub( "pos", $rpos ) ;
  echo "\n" ;
  debugSub( "wData", $wData ) ;
  echo "\n" ;
  debugSub( "wDataMst", $wDataMst ) ;
  echo "\n" ;
  debugSub( "cData", $cData ) ;
  echo "\n" ;
  debugSub( "cDataMst", $cDataMst ) ;
  echo "\n" ;
*/

  return ;
}

/* debug sub */
function debugSub( $name, $arr) {
  echo "$name{ " ;
  foreach( $arr as $k => $v ) {
    if( is_array($v) ) {
      debugSub($k, $v) ;
    } else {
      echo "$k=>$v," ;
    }
  }
  echo " }," ;
  return ;
}
?>
