<?php
include "config.php" ;
include "master.php" ;
include "relay.php" ;
include "common.php" ;

function echoTitle( $data_udp, $localhost, $listenPort, $datafile ) {
  printf( "-----------------------------\n" ) ;
  printf( "%s %s\n", PGNAME, VERSION ) ;
  printf( "-----------------------------\n" ) ;
  if( $data_udp ) {
    printf( "listening on %s:%s\n", $localhost, $listenPort ) ;
    printf( "ready to go\n" ) ;
  } else {
    printf( "reading from file: %s\n", $datafile ) ;
  }
  printf( "\n" ) ;
}

function createRelaySockets( $list ) {
  // create relay socket
  $sockSend = array() ;
  for( $i=0 ; $i<sizeof($list) ; $i++ ) {
    $sockSend[] = socket_create( AF_INET, SOCK_DGRAM, SOL_UDP ) ;
  }

  return $sockSend ;
}

function execNekometry( $pp, $data_udp, $savefile ) {
  global $pp, $phpCommand ;

  if( is_resource($pp) ) {
    proc_close( $pp ) ;
  }
  printf( "%s starting the Nekometry...", nowDate() ) ;

  // exec nekometry
  $descriptorspec = array(
    0 => array( "pipe", "r" ),
    1 => array( "pipe", "w" ),
  ) ;
  $cmd = sprintf( "%s nekometry.php", $phpCommand ) ;
  $pp = proc_open( $cmd, $descriptorspec, $pipes, NULL, NULL ) ;

  if( is_resource($pp) ) {
    // send parameter
    $param = sprintf( "%d,%s\n", $data_udp, $savefile ) ;
    fwrite( $pipes[0], $param ) ;
    printf( "done.\n" ) ;
  } else {
    printf( "failed.\n" ) ;
  }
  return array( $pp, $pipes ) ;
}

function checkPipeStat( $pipe ) {
  $stat = fstat( $pipe ) ;
  return array( "mtime" => $stat["mtime"], "size" => $stat["size"] ) ;
}

function sig_handler( $signo ) {
  global $pp, $pipes ;

  switch( $signo ) {
    case SIGINT:
    case SIGHUP:
    case SIGTERM:
      fwrite( $pipes[0], makeCommand(CMD_BYE) ) ;
      sleep( 1 );
      printf( "%s", fread( $pipes[1], 4096 ) ) ;

      fclose( $pipes[0] ) ;
      fclose( $pipes[1] ) ;
      proc_terminate( $pp ) ;
      die() ;
  }
}

// check arguments
$datafile = "" ;
$savefile = "" ;
$isResume = FALSE ;
$read_multi = 1 ;
$skip = 0 ;
$data_udp = TRUE ;
if( array_key_exists("3", $argv ) ) {
  $datafile = $argv[1] ;
  $read_multi = $argv[2] ;
  $skip = $argv[3] ;

  if( !file_exists( $datafile ) ) {
    die() ;
  }
  if( $read_multi < 0 || $read_multi > 100 ) {
    die() ;
  }
  if( $skip < 0 || $skip > 648000 ) {
    die() ;
  }
  $data_udp = FALSE ;
} elseif( array_key_exists("1", $argv ) ) {
  $savefile = $argv[1] ;
  if( !file_exists( $savefile ) ) {
    die() ;
  }
  $isResume = TRUE ;
}

// create UDP socket
if( $data_udp ) {
  $sockRcv = socket_create( AF_INET, SOCK_DGRAM, SOL_UDP ) ;
  socket_bind( $sockRcv, $localhost, $listenPort ) ;
//  $timeout = array( 'sec' => 1, 'usec' => 0 ) ;
//  socket_set_option( $sockRcv, SOL_SOCKET, SO_RCVTIMEO, $timeout ) ;
} else {
  $fpi = fopen( $datafile, "r" ) ;
  if( $skip > 0 ) {
    fseek( $fpi, $skip * PACKET_SIZE_T ) ;
    if( feof($fpi) ) {
      die() ;
    }
  }
}

/*
pcntl_signal( SIGINT, "sig_handler" ) ;
pcntl_signal( SIGHUP, "sig_handler" ) ;
pcntl_signal( SIGTERM, "sig_handler" ) ;
*/

echoTitle( $data_udp, $localhost, $listenPort, $datafile ) ;

// open nekometry
$pp = NULL ;
list( $pp, $pipes ) = execNekometry( $pp, $data_udp, $savefile ) ;
sleep( 1 ) ;
$p1stat = checkPipeStat( $pipes[1] ) ;

if( is_resource($pp) ) {
  if( $isResume ) {
    printf( "%s resume session from %s\n", nowDate(), $savefile ) ;
  }
  $fpo = NULL ;
  $prev_time = microtime( true ) ;
  $prev_rt = 0 ;
  $prev_loss = 0 ;
  $last_receive = 0 ;
  $config_update = filemtime( "relay.php" ) ;
  $sockSend = createRelaySockets( $relayList ) ;

  // main loop
  while( true ) {
    // sub loop
    for( $j=0 ; $j<300 ; $j++ ) {
      $data = "" ;
      // recieve UDP
      if( $data_udp ) {
        $sz = @socket_recvfrom( $sockRcv, $buf, 4096, 0, $from, $port ) ;
      } else {
        if( !feof($fpi) ) {
          $buf = fread( $fpi, PACKET_SIZE_T ) ;
        } else {
          fclose( $fpi ) ;
          break 2 ;
        }
      }

      // commands
      if( isCommand($buf) ) {
        $cmd = substr( $buf, 0, PACKET_SIZE_C ) ;
        if( strcmp($cmd, CMD_BYE) == 0 ) {
          // bye
          break 2 ;
        }
        fwrite( $pipes[0], $buf ) ;
        continue ;
      }

      // telemetry data
      $sz = strlen($buf) ;
      if( $sz == PACKET_SIZE_T ) {
        // unpack and extract
        $ud = unpack($UNPACK_FORMAT, $buf) ;
        $wData = getWData( $ud ) ;
        $rt = getArray( $wData,"m_time", 0 ) ;
        $era = getArray( $wData,"m_era", 0 ) ;
        if( $era >= 1980 ) {
          // wait ;
          if( !$data_udp ) {
            if( $prev_rt != 0 ) {
              $time = microtime( TRUE ) ;
              $rt_diff = ($rt - $prev_rt) * 1000000.0 ;
              $ct_diff = ($time - $prev_time) * 1000000.0 ;
              if( $read_multi > 0 ) {
                $stime = $rt_diff / $read_multi - $ct_diff ;
                $stime_adj = $stime - $prev_loss ;
                if( $stime_adj > 0 ) {
                  $time = microtime( TRUE ) ;
                  usleep( floor($stime_adj) ) ;
                  $ct_sleep = (microtime( TRUE ) - $time) * 1000000.0 ;
                  $prev_loss = $ct_sleep - $stime_adj ;
                } else {
                  $prev_loss = abs( $stime_adj ) ;
                }
              }
            }
            $prev_rt = $rt ;
            $prev_time = microtime( TRUE ) ;
          }

          // relay
          for( $i=0 ; $i<sizeof($relayList) ; $i++ ) {
            $ip = getArray( $relayList[$i], 'ip' ) ;
            $port = getArray( $relayList[$i], 'port' ) ;
            if( $ip !=0 && $port != 0 ) {
              if( !socket_sendto( $sockSend[$i], $buf, strlen($buf), 0, $ip, $port ) ) {
                printf( "WARNING: [%s:%s] socket error.\n", $ip, $port ) ;
                socket_close( $sockSend[$i] ) ;
                $sockSend[$i] = socket_create( AF_INET, SOCK_DGRAM, SOL_UDP ) ;
              }
            }
          }
          fwrite( $pipes[0], $buf ) ;
        }
      }

      // check nekometry output
      $tmp = checkPipeStat( $pipes[1] ) ;
      if( $tmp["mtime"] > $p1stat["mtime"] || $tmp["size"] > 0 ) {
        printf( "%s", fread( $pipes[1], 4096 ) ) ;
        $p1stat = $tmp ;
      }

    }

    // check nekometry stat
    $stat = proc_get_status( $pp ) ;
    if( $stat !== FALSE ) {
      if( $stat["running"] === FALSE ) {
        printf( "%s oops! something happened to the Nekometry.\n", nowDate() ) ;
        list( $pp, $pipes ) = execNekometry( $pp, $data_udp, $savefile ) ;
        $p1stat = checkPipeStat( $pipes[1] ) ;
        if( !is_resource($pp) ) {
          break ;
        }
      }
    }

    // reload config
    clearstatcache() ;
    $tmp = filemtime( "relay.php" ) ;
    if( $config_update != $tmp ) {
      printf( "%s reload relay settings.\n", nowDate() ) ;
      $config_update = $tmp ;
      // close relay socket
      for( $i=0 ; $i<sizeof($relayList) ; $i++ ) {
        socket_close( $sockSend[$i] ) ;
      }
      include "relay.php" ;
      $sockSend = createRelaySockets( $relayList ) ;
    }
  }
}

// close relay socket
for( $i=0 ; $i<sizeof($relayList) ; $i++ ) {
  socket_close( $sockSend[$i] ) ;
}
if( $data_udp ) {
  socket_close( $sockRcv ) ;
}

// wait player
$cmd = CMD_BYE ;
printf( "\nsave session data?(y/n):\n" ) ;
$buf = fgets( STDIN, 4096 ) ;
$c = substr( $buf, 0, 1 ) ;
if( strcmp($c,'y')==0 || strcmp($c,'Y')==0 ) {
  $cmd = CMD_BYE_S ;
}
fwrite( $pipes[0], makeCommand($cmd) ) ;
sleep( 1 );
printf( "%s", fread( $pipes[1], 4096 ) ) ;

fclose( $pipes[0] ) ;
fclose( $pipes[1] ) ;
if( is_resource($pp) ) {
  proc_close( $pp ) ;
}
?>
